﻿using System;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Collections.Generic;
using System.Threading;

namespace DataServiceGraphTest.Web
{
    public class RandomDataGenerator
    {
        static DateTime _previousDateTime = new DateTime(1982, 5, 27);

        public List<RandomDataModel> GetNewData()
        {
            var data = new List<RandomDataModel>();
            var rand = new Random((int)DateTime.Now.Ticks);
            for (int i = 0; i < 100; ++i)
            {
                var datetime =  _previousDateTime + RandomDataService.UpdateRate;
                data.Add(new RandomDataModel() 
                    { 
                        Value = (rand.NextDouble() * 100), 
                        Datetime =  datetime 
                    });
                _previousDateTime = datetime;
            }

            return data;
        }
    }

    [ServiceContract]
    public interface IRandomDataNotification
    {
        [OperationContract(IsOneWay = true)]
        void NotifyRandomData(List<RandomDataModel> randomData);

        [OperationContract(IsOneWay = true)]
        void NotifySummaryData(List<RandomDataSummaryModel> summaryData);
    }

    [ServiceContract(Namespace = "", CallbackContract = typeof(IRandomDataNotification))]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
    public class RandomDataService
    {
        public static readonly TimeSpan UpdateRate = new TimeSpan(0, 0, 0, 0, 1000);
        private const int decimationInterval = 10;

        public RandomDataService()
        {
            
        }

        Timer _timer;
        RandomDataGenerator _randomDataGenerator = new RandomDataGenerator();

        SynchronizedCollection<IRandomDataNotification> Clients
        {
            get
            {
                if (_timer == null)
                {
                    _timer = new Timer(new TimerCallback(UpdateClients), null, new TimeSpan(), UpdateRate);
                }

                return _clients;
            }
        }
        SynchronizedCollection<IRandomDataNotification> _clients = 
            new SynchronizedCollection<IRandomDataNotification>();

        [OperationContract]
        public void Subscribe()
        {
            Clients.Add(OperationContext.Current.GetCallbackChannel<IRandomDataNotification>());
        }

        List<RandomDataModel> _cache = new List<RandomDataModel>();

        public void UpdateClients(object o)
        {
            var newData = _randomDataGenerator.GetNewData();
            _cache.AddRange(newData);

            List<RandomDataSummaryModel> summaryData = new List<RandomDataSummaryModel>();

            while(_cache.Count >= decimationInterval)
            {
                var stats = GetCacheStats();
                stats.DecimationInterval = decimationInterval;
                summaryData.Add(stats);
                _cache.RemoveRange(0, decimationInterval);
            }

            foreach (IRandomDataNotification client in Clients)
            {
                client.NotifyRandomData(newData);

                if(summaryData.Count > 0)
                {
                    client.NotifySummaryData(summaryData);
                }
            }
        }


        RandomDataSummaryModel GetCacheStats()
        {
            var stats = new RandomDataSummaryModel();

            stats.Open = _cache[0].Value;
            stats.Close = _cache[decimationInterval-1].Value;

            stats.Datetime = _cache[decimationInterval - 1].Datetime;

            double max = _cache[0].Value;
            double min = _cache[0].Value;

            for (int i = 1; i < decimationInterval; ++i)
            {
                if (_cache[i].Value > max)
                {
                    max = _cache[i].Value;
                }

                if (_cache[i].Value < min)
                {
                    min = _cache[i].Value;
                }
            }

            stats.High = max;
            stats.Low = min;

            return stats;
        }
    }
}
